module.exports = function (grunt) {
	var backendModules = ['../../teknisa/cus'];
	var directories = ['utilities', 'repositories', 'services', 'controllers'];
	var subModules = ['relatorio', 'parametros', 'operacional'];
	var libs = [
		"./bower_components/zh-reports/dist/zhReports.js",
		"./bower_components/zh-preferences/dist/preferences.js",
		"./bower_components/zh-filter-preferences/dist/filter-preferences.js",
        "./bower_components/zh-layout-preferences/dist/layout-preferences.js",
		"./bower_components/zh-grid-configuration/dist/zh-grid-configuration.js",
        "./bower_components/zh-view-preferences/dist/view-preferences.js",
        "./bower_components/zh-select-autocomplete/dist/zh-select-autocomplete.js",
		"./bower_components/zh-interval-component/dist/zh-interval-component.js",
		"./bower_components/zh-grid-navigation/dist/GridNavigationLibrary.js",
		"./bower_components/zh-cache-bypass/src/CacheBypass.js",
		"./bower_components/zhuaclfrontend/assets/dist/main.js",
		"./bower_components/zhulog/dist/main.js",
		"./bower_components/tek-libraries/dist/main.js",
		"./bower_components/zh-reports/dist/zhReports.js"
	];

	grunt.initConfig({
		concat: {
			js: {
				options: {
					process: function (src, filepath) {
						return '// FILE: ' + filepath + '\n' + src + '\n';
					}
				},
				src: getJSPaths(),
				dest: './assets/js/app.js'
			},
			css: {
				options: {
					process: function (src, filepath) {
						return '/* FILE: ' + filepath + ' */\n' + src + '\n';
					}
				},
				src: getCSSPaths(),
				dest: './assets/css/app.css'
			},
			libs: {
				options: {
					process: function (src, filepath) {
						return '// FILE: ' + filepath + '\n' + src + '\n';
					}
				},
				src: libs,
				dest: './assets/js/libs.js'
			}
		},
		uglify: {
			options: {
				mangle: false,
				compress: false,
				report: 'min',
				// the banner is inserted at the top of the output
				banner: '/*! <%= grunt.template.today("dd-mm-yyyy") %> */\n'
			},
			dist: {
				files: {
					'./assets/js/app.min.js': getJSPaths(),
                    './assets/js/libs.min.js': libs,
                    './config/zeedhiConfig.min.js': 'config/zeedhiConfig.js'
                }
			}
		},
		jshint: {
			all: getJSPaths().concat(["GruntFile.js"]),
			options: {
				evil: true,
				esversion: 6
			}
		},
		watch: {
			js: {
				files: getJSPaths().concat(["GruntFile.js"]),
				tasks: ['jshint', 'concat:js']
			},
			css: {
				files: getCSSPaths(),
				tasks: ['concat:css']
			}
		},
		cssmin: {
			dist: {
				files: {
	                    "assets/css/app.min.css": getCSSPaths(),
	                    "bower_components/zeedhi-frontend/assets/fonts/font-zeedhi/stylesheet.min.css": "bower_components/zeedhi-frontend/assets/fonts/font-zeedhi/stylesheet.css",
	                    "bower_components/zh-preferences/dist/style/style.min.css": "bower_components/zh-preferences/dist/style/style.css",
	                    "bower_components/zh-preferences/dist/style/colors/styles-tecfood.min.css":"bower_components/zh-preferences/dist/style/colors/styles-tecfood.css",
			            "bower_components/zh-grid-configuration/dist/zh-grid-configuration.min.css": "bower_components/zh-grid-configuration/dist/zh-grid-configuration.css",
			            "bower_components/zh-select-autocomplete/dist/style/colors/styles-tecfood.min.css":"bower_components/zh-select-autocomplete/dist/style/colors/styles-tecfood.css",
			            "bower_components/zh-reports/dist/style/style.min.css":"bower_components/zh-reports/dist/style/style.css",
			            "bower_components/zh-reports/dist/style/colors/styles-tecfood.min.css":"bower_components/zh-reports/dist/style/colors/styles-tecfood.css",
			            "bower_components/zh-grid-navigation/dist/style.min.css": "bower_components/zh-grid-navigation/dist/style.css"
	                }
			}
		},
		// NÃO COLOCAR PROPRIEDADE PARA SUBSTIUIR IDS
		zhIdGenerator: {
			options: {
				jsonPath: getJSONPaths(),
				menuPath: ["./assets/json/menu.json"]
			}
		},
        buildAclStruct : {
            options: {
                menuConfigFile: "./menuConfig.json"
            }
        },
		buildEnvironment: {
			development: {
				configFile: "./config/devEnvironment.json"
			},
			production: {
				configFile: "./config/prodEnvironment.json"
			},
			test: {
				configFile: "./config/testEnvironment.json"
			},
			srcFolders: ["../backend/config/", "./config", "./", "./assets", "./assets/json", "../../teknisa/cus/reports"]
		},
		zhWords: {
			options: {
				serverUrl: "https://translate.teknisa.com/backend/index.php",
				organization: "20",
				project: "2235",
				repository: "1241",
				birtBasePath: "../../teknisa/cus"
			}
		}
	});

	function getJSPaths() {
		var paths = [];
		directories.forEach(function (directory) {
			paths.push('./assets/js/' + directory + '/**/*.js');
			subModules.forEach(function (subModule) {
				paths.push('./assets/modules/' + subModule + '/assets/js/' + directory + '/**/*.js');
			});
		});
		return paths;
	}

	function getCSSPaths() {
		var paths = [];
		paths.push('./assets/css/**/*.css');
		paths.push('!./assets/css/app.css');
		subModules.forEach(function (subModule) {
			paths.push('./assets/modules/' + subModule + '/assets/css/**/*.css');
			paths.push('!./assets/modules/' + subModule + '/assets/css/app.css');
		});
		return paths;
	}

	function getJSONPaths() {
		var paths = ["./assets/json"];
		subModules.forEach(function (subModule) {
			paths.push("./assets/modules/" + subModule + "/assets/json");
		});
		return paths;
	}

	function buildFileContent(srcFiles, filePath, directory) {
		var parts = filePath.split('/');
		parts.shift();
		parts.shift();
		var filename = parts[parts.length - 1];
		var extension = filename.split('.').pop().toUpperCase();
		if (extension == 'JSON' || extension == 'XML') {
			if (directory == 'datasources') {
				filename = parts.pop();
				filename = filename.replace(/\.[^/.]+$/, "");
				srcFiles[directory][filename] = './../../../' + parts.join('/');
			} else {
				srcFiles[directory].push('/../../' + parts.join('/'));
			}
		}
	}

	grunt.registerTask('backendStep', 'build backend config files', function () {
		var srcFiles = {
			config: [],
			datasources: {}
		};
		var directories = ['config', 'datasources'];
		var dest = {
			config: '../backend/config/servicesMapping.json',
			datasources: '../backend/config/dataSourcesMapping.json'
		};
		directories.forEach(function (directory) {
			backendModules.forEach(function (module) {
				var moduleParts = module.split('/');
				var moduleName = moduleParts[moduleParts.length - 1].toUpperCase();
				var subModules = grunt.file.expand({
					filter: 'isDirectory'
				}, module + '/src/Teknisa/' + moduleName + '/Modules/*');

				subModules.forEach(function (subModule) {
					grunt.file.expand({
						filter: 'isFile'
					}, subModule + '/' + directory + '/**/*').forEach(function (filePath) {
						buildFileContent(srcFiles, filePath, directory);
					});
				});

				grunt.file.expand({
					filter: 'isFile'
				}, module + '/' + directory + '/**/*').forEach(function (filePath) {
					buildFileContent(srcFiles, filePath, directory);
				});
			});
		});
		grunt.file.write(dest.config, JSON.stringify(srcFiles.config, null, 2));
		grunt.file.write(dest.datasources, JSON.stringify(srcFiles.datasources, null, 2));
	});

	grunt.loadNpmTasks('grunt-contrib-jshint');
	grunt.loadNpmTasks('grunt-contrib-uglify');
	grunt.loadNpmTasks('grunt-contrib-concat');
	grunt.loadNpmTasks('grunt-contrib-watch');
	grunt.loadNpmTasks('grunt-contrib-cssmin');
	grunt.loadNpmTasks('grunt-zh-id-generator');
	grunt.loadNpmTasks('grunt-zhuacl-structure');
	grunt.loadNpmTasks('grunt-zh-environment-builder');
	grunt.loadNpmTasks('grunt-zh-words');

	grunt.registerTask('default', ['zhWords', 'jshint', 'backendStep', 'cssmin', 'buildAclStruct', 'buildEnvironment',"uglify"]);
	grunt.registerTask('mergeBranch', ['zhIdGenerator', 'buildAclStruct']);
	grunt.registerTask('final', ['cssmin', 'uglify']);
}; 